/*
 * server.vala - dbus server implementation, parameter validation
 *
 * Authored by Michael 'Mickey' Lauer <mlauer@vanille-media.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

//===========================================================================
using GLib;
using CONST;

//===========================================================================
[DBus (name = "org.freesmartphone.GSM.MUX")]
public errordomain DBusMuxerError
{
    ChannelTaken,
    NoChannel,
    NoSession,
    SessionAlreadyOpen,
    SessionOpenError
}

//===========================================================================
[DBus (name = "org.freesmartphone.GSM.MUX")]
public abstract interface OrgFreesmartphoneGsmMux
{
    public abstract string GetVersion() throws DBus.Error, DBusMuxerError;
    public abstract bool HasAutoSession() throws DBus.Error, DBusMuxerError;
    public abstract void OpenSession( bool advanced, int framesize, string port, int portspeed ) throws DBus.Error, DBusMuxerError;
    public abstract void CloseSession() throws DBus.Error, DBusMuxerError;
    public abstract void AllocChannel( string name, int channel, out string path, out int allocated_channel ) throws DBus.Error, DBusMuxerError;
    public abstract void ReleaseChannel( string name ) throws DBus.Error, DBusMuxerError;
    public abstract void SetWakeupThreshold( uint seconds, uint waitms ) throws DBus.Error, DBusMuxerError;
    public abstract void SetStatus( int channel, string status ) throws DBus.Error, DBusMuxerError;
    public signal void Status( int channel, string status );
    public abstract void TestCommand( uint8[] data ) throws DBus.Error, DBusMuxerError;
}

//===========================================================================
public class Server : OrgFreesmartphoneGsmMux, Object
{
    DBus.Connection conn;
    dynamic DBus.Object dbus;

    Gsm0710mux.Manager manager;

    public Server()
    {
        try
        {
            debug( "DBus-Server: created" );
            conn = DBus.Bus.get( DBus.BusType.SYSTEM );
            dbus = conn.get_object( DBUS_BUS_NAME, DBUS_OBJ_PATH, DBUS_INTERFACE );
        } catch ( DBusMuxerError e )
        {
            error( "DBus-Server: %s", e.message );
        }

        manager = new Gsm0710mux.Manager();
    }

    ~Server()
    {
        debug( "DBus-Server: destructed" );
    }

    //
    // DBus API
    //

    public string GetVersion() throws DBus.Error, DBusMuxerError
    {
        return manager.getVersion();
    }

    public bool HasAutoSession() throws DBus.Error, DBusMuxerError
    {
        return manager.hasAutoSession();
    }

    public void OpenSession( bool advanced, int framesize, string port, int portspeed ) throws DBus.Error, DBusMuxerError
    {
        try
        {
            manager.openSession( advanced, framesize, port, portspeed );
        }
        catch ( Gsm0710mux.MuxerError e )
        {
            if ( e is Gsm0710mux.MuxerError.SessionAlreadyOpen )
                throw new DBusMuxerError.SessionAlreadyOpen( e.message );
            else if ( e is Gsm0710mux.MuxerError.SessionOpenError )
                throw new DBusMuxerError.SessionOpenError( e.message );
            else
                throw e;
        }
    }

    public void CloseSession() throws DBus.Error, DBusMuxerError
    {
        try
        {
            manager.closeSession();
        }
        catch ( Gsm0710mux.MuxerError e )
        {
            if ( e is Gsm0710mux.MuxerError.NoSession )
                throw new DBusMuxerError.NoSession( e.message );
        }
    }

    public void AllocChannel( string name, int channel, out string path, out int allocated_channel ) throws DBus.Error, DBusMuxerError
    {
        debug( "AllocChannel requested for name %s, requested channel %d", name, channel );

        var ci = Gsm0710mux.ChannelInfo();
        ci.type = Gsm0710mux.ChannelType.PTY;
        ci.consumer = name;
        ci.number = channel;

        try
        {
            manager.allocChannel( ref ci );
        }
        catch ( Gsm0710mux.MuxerError e )
        {
            if ( e is Gsm0710mux.MuxerError.ChannelTaken )
                throw new DBusMuxerError.ChannelTaken( e.message );
            else if ( e is Gsm0710mux.MuxerError.NoChannel )
                throw new DBusMuxerError.NoChannel( e.message );
            else throw e;
        }

        path = ci.transport;
        allocated_channel = ci.number;
    }

    public void ReleaseChannel( string name ) throws DBus.Error, DBusMuxerError
    {
        try
        {
            manager.releaseChannel( name );
        }
        catch ( Gsm0710mux.MuxerError e )
        {
            if ( e is Gsm0710mux.MuxerError.NoSession )
                throw new DBusMuxerError.ChannelTaken( e.message );
            else
                throw e;
        }
    }

    public void SetWakeupThreshold( uint seconds, uint waitms ) throws DBus.Error, DBusMuxerError
    {
        manager.setWakeupThreshold( seconds, waitms );
    }

    public void SetStatus( int channel, string status ) throws DBus.Error, DBusMuxerError
    {
        try
        {
            manager.setStatus( channel, status );
        }
        catch ( Gsm0710mux.MuxerError e )
        {
            if ( e is Gsm0710mux.MuxerError.NoSession )
                throw new DBusMuxerError.ChannelTaken( e.message );
            else
                throw e;
        }
    }

    public void TestCommand( uint8[] data ) throws DBus.Error, DBusMuxerError
    {
        manager.testCommand( data );
    }

}
